//
//
// week7.c
//
// Simon says
//
// Eyal Perry
// 10/20/19
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define LED (1 << PA3)
#define BUTTON (1 << PA2)

#define BUTTON_UP 0
#define BUTTON_DOWN 1

#define MAX_LEVEL 8

#define STATUS_INIT 0
#define STATUS_WAITING_FOR_INPUT 1

// for 20MHz, 5000 is 256ms
#define TAP_SHORT_MAX 5000
#define TAP_LONG_MIN 5000

static unsigned int game_level = 1;
static char button_status = BUTTON_UP;
static char game_status = STATUS_INIT;
static unsigned int sequence = 0;
static unsigned int count = 0;
static unsigned int last_timer = 0;

int button_pressed(void) {
  if (PINA & BUTTON) {
    // button is up
    if (button_status & BUTTON_DOWN) {
      button_status = BUTTON_UP;
      last_timer = TCNT1;
      return 1;
    }
  } else {
    // button is down
    if (!button_status) {
      // first press
      TCNT1 = 0;
    }
    button_status = BUTTON_DOWN;
  }
  return 0;
}

void play_sequence() {
  sequence = last_timer;

  _delay_ms(200);

  for (int i = 0; i < game_level; i++) {
    PORTA |= LED;
    _delay_ms(150);
    if (sequence & (1 << i))
      _delay_ms(350);
    PORTA &= ~LED;
    if (i + 1 < game_level) {
      _delay_ms(500);
    }
  }

  game_status = STATUS_WAITING_FOR_INPUT;
  count = 0;
}

void lose(void) {
  for (int i = 0; i < 15; i++) {
    PORTA |= LED;
    _delay_ms(50);
    PORTA &= ~LED;
    _delay_ms(50);
  }
}

void win(void) {
  PORTA |= LED;
  _delay_ms(3000);
}

int main(void) {
  //
  // set clock divider to /1
  // took this code from Neil. not sure what it does but it seems important to make_delay_ms work properly
  //
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

  // set LED as output
  DDRA |= LED;
  // set button as input
  DDRA &= ~BUTTON;
  // pull LED and button to high
  PORTA |= LED;
  PORTA |= BUTTON;

  // set timer prescaler to /1024
  // assuming a clock speed of 20Mhz, a 16 bit timer can count until:
  // 1 / (20 / 1024) * 65536 = 3.3sec~
  // good enough for us, since i only use to to measure taps
  TCCR1B = (1<<CS10) | (1<<CS12);
  TCNT1 = 0;
  //
  // main loop
  //
  while (1) {
    if (button_pressed()) {
      if (game_status == STATUS_INIT) {
        play_sequence();
      } else if (game_status == STATUS_WAITING_FOR_INPUT) {
        // check if the press is correct
        if (((sequence & (1 << count)) && last_timer > 5000) ||
            (!(sequence & (1 << count)) && last_timer < 5000)) {
          // success!
          count++;

          // if reached the count, go to the next level
          if (count >= game_level) {
            game_level++;

            if (game_level <= MAX_LEVEL) {
              play_sequence();
            } else {
              // game over!
              win();
              game_level = 1;
              game_status = STATUS_INIT;
            }
          }
        } else {
          lose();
          game_level = 1;
          game_status = STATUS_INIT;
        }
      }
    }
    PORTA &= ~LED;
  }
}
